@@ -96,8 +96,9 @@ class ProgressBarBase(Callback):
         if trainer.fast_dev_run and trainer.val_dataloaders is not None:
             total_val_batches = len(trainer.val_dataloaders)
         elif not self.trainer.disable_validation:
-            is_val_epoch = (trainer.current_epoch + 1) % trainer.check_val_every_n_epoch == 0
+            is_val_epoch = trainer.current_epoch % trainer.check_val_every_n_epoch == 0
             total_val_batches = trainer.num_val_batches if is_val_epoch else 0
+            total_val_batches = sum(total_val_batches)
         return total_val_batches
 
     @property
@@ -111,6 +112,7 @@ class ProgressBarBase(Callback):
             total_test_batches = len(self.trainer.test_dataloaders)
         else:
             total_test_batches = self.trainer.num_test_batches
+            total_test_batches = sum(total_test_batches)
         return total_test_batches
 
     def disable(self):
@@ -317,13 +319,13 @@ class ProgressBar(ProgressBarBase):
         total_batches = total_train_batches + total_val_batches
         if not self.main_progress_bar.disable:
             self.main_progress_bar.reset(convert_inf(total_batches))
-        self.main_progress_bar.set_description(f'Epoch {trainer.current_epoch + 1}')
+        self.main_progress_bar.set_description(f'Epoch {trainer.current_epoch}')
 
     def on_batch_end(self, trainer, pl_module):
         super().on_batch_end(trainer, pl_module)
         if self.is_enabled and self.train_batch_idx % self.refresh_rate == 0:
             self.main_progress_bar.update(self.refresh_rate)
-            self.main_progress_bar.set_postfix(**trainer.progress_bar_dict)
+            self.main_progress_bar.set_postfix(trainer.progress_bar_dict)
 
     def on_validation_start(self, trainer, pl_module):
         super().on_validation_start(trainer, pl_module)
@@ -338,7 +340,7 @@ class ProgressBar(ProgressBarBase):
 
     def on_validation_end(self, trainer, pl_module):
         super().on_validation_end(trainer, pl_module)
-        self.main_progress_bar.set_postfix(**trainer.progress_bar_dict)
+        self.main_progress_bar.set_postfix(trainer.progress_bar_dict)
         self.val_progress_bar.close()
 
     def on_train_end(self, trainer, pl_module):
