@@ -125,7 +125,7 @@ class LightningLoggerBase(ABC):
         """
         agg_step, metrics_to_log = self._aggregate_metrics(metrics=metrics, step=step)
 
-        if metrics_to_log is not None:
+        if metrics_to_log:
             self.log_metrics(metrics=metrics_to_log, step=agg_step)
 
     @abstractmethod
@@ -292,6 +292,41 @@ class LoggerCollection(LightningLoggerBase):
         return '_'.join([str(logger.version) for logger in self._logger_iterable])
 
 
+class DummyExperiment(object):
+    """ Dummy experiment """
+    def nop(*args, **kw):
+        pass
+
+    def __getattr__(self, _):
+        return self.nop
+
+
+class DummyLogger(LightningLoggerBase):
+    """ Dummy logger for internal use. Is usefull if we want to disable users
+        logger for a feature, but still secure that users code can run """
+    def __init__(self):
+        super().__init__()
+        self._experiment = DummyExperiment()
+
+    @property
+    def experiment(self):
+        return self._experiment
+
+    def log_metrics(self, metrics, step):
+        pass
+
+    def log_hyperparams(self, params):
+        pass
+
+    @property
+    def name(self):
+        pass
+
+    @property
+    def version(self):
+        pass
+
+
 def merge_dicts(
         dicts: Sequence[Mapping],
         agg_key_funcs: Optional[Mapping[str, Callable[[Sequence[float]], float]]] = None,
