@@ -10,9 +10,11 @@ from typing import Optional, Dict, Any, Union
 try:
     import mlflow
     from mlflow.tracking import MlflowClient
+    _MLFLOW_AVAILABLE = True
 except ImportError:  # pragma: no-cover
-    raise ImportError('You want to use `mlflow` logger which is not installed yet,'  # pragma: no-cover
-                      ' install it with `pip install mlflow`.')
+    mlflow = None
+    MlflowClient = None
+    _MLFLOW_AVAILABLE = False
 
 from pytorch_lightning import _logger as log
 from pytorch_lightning.loggers.base import LightningLoggerBase
@@ -54,11 +56,16 @@ class MLFlowLogger(LightningLoggerBase):
         tags: A dictionary tags for the experiment.
 
     """
+
     def __init__(self,
                  experiment_name: str = 'default',
                  tracking_uri: Optional[str] = None,
                  tags: Optional[Dict[str, Any]] = None,
                  save_dir: Optional[str] = None):
+
+        if not _MLFLOW_AVAILABLE:
+            raise ImportError('You want to use `mlflow` logger which is not installed yet,'
+                              ' install it with `pip install mlflow`.')
         super().__init__()
         if not tracking_uri and save_dir:
             tracking_uri = f'file:{os.sep * 2}{save_dir}'
