@@ -1,4 +1,12 @@
 """
+.. testsetup:: *
+
+    import os
+    from pytorch_lightning.trainer.trainer import Trainer
+    from pytorch_lightning.core.lightning import LightningModule
+    from pytorch_lightning.utilities.seed import seed_everything
+
+
 Once you've organized your PyTorch code into a LightningModule,
 the Trainer automates everything else.
 
@@ -23,8 +31,6 @@ This is the basic use of the trainer:
 
 .. code-block:: python
 
-    from pytorch_lightning import Trainer
-
     model = MyLightningModule()
 
     trainer = Trainer()
@@ -54,7 +60,7 @@ main.py file this way
 
         main(args)
 
-So you can run it like so:distributed_backend
+So you can run it like so:
 
 .. code-block:: bash
 
@@ -101,6 +107,11 @@ Use it to do whatever!
     out = pretrained_model(x)
     api_write({'response': out}
 
+
+You may wish to run the model on a variety of devices. Instead of moving the data
+manually to the correct device, decorate the forward method (or any other method you use for inference)
+with :func:`~pytorch_lightning.core.decorators.auto_move_data` and Lightning will take care of the rest.
+
 ------------
 
 Reproducibility
@@ -109,9 +120,9 @@ Reproducibility
 To ensure full reproducibility from run to run you need to set seeds for pseudo-random generators,
 and set ``deterministic``` flag in ``Trainer``.
 
-.. code-block:: python
+Example::
 
-    from pytorch-lightning import Trainer, seed_everything
+    from pytorch_lightning import Trainer, seed_everything
 
     seed_everything(42)
     # sets seeds for numpy, torch, python.random and PYTHONHASHSEED.
@@ -128,7 +139,7 @@ accumulate_grad_batches
 ^^^^^^^^^^^^^^^^^^^^^^^
 Accumulates grads every k batches or as set up in the dict.
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer (no accumulation)
     trainer = Trainer(accumulate_grad_batches=1)
@@ -158,7 +169,7 @@ auto_scale_batch_size
 Automatically tries to find the largest batch size that fits into memory,
 before any training.
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer (no scaling of batch size)
     trainer = Trainer(auto_scale_batch_size=None)
@@ -171,7 +182,7 @@ auto_lr_find
 Runs a learning rate finder algorithm (see this `paper <https://arxiv.org/abs/1506.01186>`_)
 before any training, to find optimal initial learning rate.
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer (no learning rate finder)
     trainer = Trainer(auto_lr_find=False)
@@ -238,10 +249,10 @@ Example::
     from pytorch_lightning.callbacks import Callback
 
     class PrintCallback(Callback):
-        def on_train_start(self):
+        def on_train_start(self, trainer, pl_module):
             print("Training is started!")
-        def on_train_end(self):
-            print(f"Training is done. The logs are: {self.trainer.logs}")
+        def on_train_end(self, trainer, pl_module):
+            print("Training is done.")
 
 check_val_every_n_epoch
 ^^^^^^^^^^^^^^^^^^^^^^^
@@ -262,7 +273,8 @@ Callback for checkpointing.
 
 .. code-block:: python
 
-    trainer = Trainer(checkpoint_callback=checkpoint_callback)
+    from pytorch_lightning.callbacks import ModelCheckpoint
+    trainer = Trainer(checkpoint_callback=ModelCheckpoint())
 
 Example::
 
@@ -304,7 +316,7 @@ The distributed backend to use.
 - (```ddp2```) dp on node, ddp across nodes. Useful for things like increasing
     the number of negative samples
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(distributed_backend=None)
@@ -322,6 +334,10 @@ Example::
 
 .. note:: this option does not apply to TPU. TPUs use ```ddp``` by default (over each core)
 
+See Also:
+    - `Multi-GPU training guide <multi_gpu.rst>`_
+    - `Multi-node (SLURM) guide <slurm.rst>`_
+
 early_stop_callback
 ^^^^^^^^^^^^^^^^^^^
 
@@ -334,22 +350,19 @@ early_stop_callback (:class:`pytorch_lightning.callbacks.EarlyStopping`)
 - ``None``: The default callback monitoring ``'val_loss'`` is created.
 - Default: ``None``.
 
-.. code-block:: python
-
-    trainer = Trainer(early_stop_callback=early_stop_callback)
-
-Example::
+.. testcode::
 
     from pytorch_lightning.callbacks import EarlyStopping
 
     # default used by the Trainer
-    early_stop_callback = EarlyStopping(
+    early_stop = EarlyStopping(
         monitor='val_loss',
         patience=3,
         strict=False,
         verbose=False,
         mode='min'
     )
+    trainer = Trainer(early_stop_callback=early_stop)
 
 .. note:: If ``'val_loss'`` is not found will work as if early stopping is disabled.
 
@@ -375,7 +388,7 @@ Under the hood the pseudocode looks like this:
     out = validation_step(val_batch)
     validation_epoch_end([out])
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(fast_dev_run=False)
@@ -390,11 +403,13 @@ gpus
 - or Which GPUs to train on
 - can handle strings
 
-Example::
+.. testcode::
 
     # default used by the Trainer (ie: train on CPU)
     trainer = Trainer(gpus=None)
 
+Example::
+
     # int: train on 2 gpus
     trainer = Trainer(gpus=2)
 
@@ -410,7 +425,8 @@ Example::
     # uses 8 gpus in total
     trainer = Trainer(gpus=2, num_nodes=4)
 
-.. note:: See the `multi-gpu computing guide <multi_gpu.rst>`_
+See Also:
+    - `Multi-GPU training guide <multi_gpu.rst>`_
 
 gradient_clip_val
 ^^^^^^^^^^^^^^^^^
@@ -418,17 +434,44 @@ Gradient clipping value
 
 - 0 means don't clip.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(gradient_clip_val=0.0)
 
 
-gradient_clip:
+limit_test_batches
+^^^^^^^^^^^^^^^^^^
 
-.. warning:: .. deprecated:: 0.5.0
+How much of test dataset to check.
 
-    Use `gradient_clip_val` instead. Will remove 0.8.0.
+.. testcode::
+
+    # default used by the Trainer
+    trainer = Trainer(limit_test_batches=1.0)
+
+    # run through only 25% of the test set each epoch
+    trainer = Trainer(limit_test_batches=0.25)
+
+    # run for only 10 batches
+    trainer = Trainer(limit_test_batches=10)
+
+limit_val_batches
+^^^^^^^^^^^^^^^^^
+
+How much of validation dataset to check.
+Useful when debugging or testing something that happens at the end of an epoch.
+
+.. testcode::
+
+    # default used by the Trainer
+    trainer = Trainer(limit_val_batches=1.0)
+
+    # run through only 25% of the validation set each epoch
+    trainer = Trainer(limit_val_batches=0.25)
+
+    # run for only 10 batches
+    trainer = Trainer(limit_val_batches=10)
 
 log_gpu_memory
 ^^^^^^^^^^^^^^
@@ -438,7 +481,7 @@ Options:
 - 'min_max'
 - 'all'
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(log_gpu_memory=None)
@@ -456,7 +499,7 @@ log_save_interval
 
 Writes logs to disk this often.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(log_save_interval=100)
@@ -466,62 +509,46 @@ logger
 
 `Logger <loggers.rst>`_ (or iterable collection of loggers) for experiment tracking.
 
-.. code-block:: python
-
-    Trainer(logger=logger)
-
-Example::
+.. testcode::
 
     from pytorch_lightning.loggers import TensorBoardLogger
 
     # default logger used by trainer
     logger = TensorBoardLogger(
         save_dir=os.getcwd(),
-        version=self.slurm_job_id,
+        version=1,
         name='lightning_logs'
     )
+    Trainer(logger=logger)
 
 max_epochs
 ^^^^^^^^^^
 Stop training once this number of epochs is reached
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(max_epochs=1000)
 
-max_nb_epochs:
-
-.. warning:: .. deprecated:: 0.5.0
-
-    Use `max_epochs` instead. Will remove 0.8.0.
-
 min_epochs
 ^^^^^^^^^^
 Force training for at least these many epochs
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(min_epochs=1)
 
-min_nb_epochs:
-
-.. warning:: deprecated:: 0.5.0
-    Use `min_epochs` instead. Will remove 0.8.0.
-
 max_steps
 ^^^^^^^^^
 Stop training after this number of steps
 Training will stop if max_steps or max_epochs have reached (earliest).
 
-.. code-block:: python
+.. testcode::
 
     # Default (disabled)
     trainer = Trainer(max_steps=None)
 
-Example::
-
     # Stop after 100 steps
     trainer = Trainer(max_steps=100)
 
@@ -531,13 +558,11 @@ min_steps
 Force training for at least these number of steps.
 Trainer will train model for at least min_steps or min_epochs (latest).
 
-.. code-block:: python
+.. testcode::
 
     # Default (disabled)
     trainer = Trainer(min_steps=None)
 
-Example::
-
     # Run at least for 100 steps (disable min_epochs)
     trainer = Trainer(min_steps=100, min_epochs=0)
 
@@ -546,7 +571,7 @@ num_nodes
 
 Number of GPU nodes for distributed training.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(num_nodes=1)
@@ -554,12 +579,6 @@ Example::
     # to train on 8 nodes
     trainer = Trainer(num_nodes=8)
 
-nb_gpu_nodes:
-
-.. warning:: .. deprecated:: 0.5.0
-
-    Use `num_nodes` instead. Will remove 0.8.0.
-
 num_processes
 ^^^^^^^^^^^^^
 
@@ -570,7 +589,7 @@ machine without GPUs. This is useful for debugging, but **will not** provide
 any speedup, since single-process Torch already makes effient use of multiple
 CPUs.
 
-Example::
+.. testcode::
 
     # Simulate DDP for debugging on your GPU-less laptop
     trainer = Trainer(distributed_backend="ddp_cpu", num_processes=2)
@@ -582,7 +601,7 @@ Sanity check runs n batches of val before starting the training routine.
 This catches any bugs in your validation without having to wait for the first validation check.
 The Trainer uses 5 steps by default. Turn it off or modify it here.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(num_sanity_val_steps=5)
@@ -590,15 +609,37 @@ Example::
     # turn it off
     trainer = Trainer(num_sanity_val_steps=0)
 
-nb_sanity_val_steps:
-
-.. warning:: .. deprecated:: 0.5.0
-
-    Use `num_sanity_val_steps` instead. Will remove 0.8.0.
-
 num_tpu_cores
 ^^^^^^^^^^^^^
-How many TPU cores to train on (1 or 8).
+.. warning:: .. deprecated:: 0.7.6
+
+    Use `tpu_cores` instead. Will remove 0.9.0.
+
+Example::
+
+    python -m torch_xla.distributed.xla_dist
+    --tpu=$TPU_POD_NAME
+    --conda-env=torch-xla-nightly
+    --env=XLA_USE_BF16=1
+    -- python your_trainer_file.py
+
+prepare_data_per_node
+^^^^^^^^^^^^^^^^^^^^^
+If True will call `prepare_data()` on LOCAL_RANK=0 for every node.
+If False will only call from NODE_RANK=0, LOCAL_RANK=0
+
+.. testcode::
+
+    # default
+    Trainer(prepare_data_per_node=True)
+
+    # use only NODE_RANK=0, LOCAL_RANK=0
+    Trainer(prepare_data_per_node=False)
+
+tpu_cores
+^^^^^^^^^
+- How many TPU cores to train on (1 or 8).
+- Which TPU core to train on [1-8]
 
 A single TPU v2 or v3 has 8 cores. A TPU pod has
 up to 2048 cores. A slice of a POD means you get as many cores
@@ -610,26 +651,26 @@ Your effective batch size is batch_size * total tpu cores.
 
 This parameter can be either 1 or 8.
 
-Example::
+.. testcode::
 
     # your_trainer_file.py
 
     # default used by the Trainer (ie: train on CPU)
-    trainer = Trainer(num_tpu_cores=None)
+    trainer = Trainer(tpu_cores=None)
 
     # int: train on a single core
-    trainer = Trainer(num_tpu_cores=1)
+    trainer = Trainer(tpu_cores=1)
+
+    # list: train on a single selected core
+    trainer = Trainer(tpu_cores=[2])
 
     # int: train on all cores few cores
-    trainer = Trainer(num_tpu_cores=8)
+    trainer = Trainer(tpu_cores=8)
 
     # for 8+ cores must submit via xla script with
     # a max of 8 cores specified. The XLA script
     # will duplicate script onto each TPU in the POD
-    trainer = Trainer(num_tpu_cores=8)
-
-    # -1: train on all available TPUs
-    trainer = Trainer(num_tpu_cores=-1)
+    trainer = Trainer(tpu_cores=8)
 
 To train on more than 8 cores (ie: a POD),
 submit this script using the xla_dist script.
@@ -644,29 +685,28 @@ Example::
 
 overfit_pct
 ^^^^^^^^^^^
-Uses this much data of all datasets (training, validation, test).
+
+.. warning:: .. deprecated:: 0.8.0.
+
+    Use `overfit_batches`. Will remove 1.0.0.
+
+overfit_batches
+^^^^^^^^^^^^^^^
+Uses this much data of the training set. If will use the same training set for validation and testing.
+If the training Dataloaders(shuffle=True), Lightning will automatically disable it.
+
 Useful for quickly debugging or trying to overfit on purpose.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(overfit_pct=0.0)
+    trainer = Trainer(overfit_batches=0.0)
 
-    # use only 1% of the train, test, val datasets
-    trainer = Trainer(overfit_pct=0.01)
-
-    # equivalent:
-    trainer = Trainer(
-        train_percent_check=0.01,
-        val_percent_check=0.01,
-        test_percent_check=0.01
-    )
-
-See Also:
-    - `train_percent_check`_
-    - `val_percent_check`_
-    - `test_percent_check`_
+    # use only 1% of the train set (and use the train set for val and test)
+    trainer = Trainer(overfit_batches=0.01)
 
+    # overfit on 10 of the same batches
+    trainer = Trainer(overfit_batches=10)
 
 precision
 ^^^^^^^^^
@@ -676,7 +716,7 @@ Can be used on CPU, GPU or TPUs.
 If used on TPU will use torch.bfloat16 but tensor printing
 will still show torch.float32.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(precision=32)
@@ -684,6 +724,8 @@ Example::
     # 16-bit precision
     trainer = Trainer(precision=16)
 
+Example::
+
     # one day
     trainer = Trainer(precision=8|4|2)
 
@@ -700,7 +742,7 @@ process_position
 ^^^^^^^^^^^^^^^^
 Orders the progress bar. Useful when running multiple trainers on the same node.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(process_position=0)
@@ -714,9 +756,9 @@ To profile individual steps during training and assist in identifying bottleneck
 
 See the `profiler documentation <profiler.rst>`_. for more details.
 
-Example::
+.. testcode::
 
-    from pytorch_lightning.profiler import Profiler, AdvancedProfiler
+    from pytorch_lightning.profiler import SimpleProfiler, AdvancedProfiler
 
     # default used by the Trainer
     trainer = Trainer(profiler=None)
@@ -725,12 +767,10 @@ Example::
     trainer = Trainer(profiler=True)
 
     # equivalent to profiler=True
-    profiler = Profiler()
-    trainer = Trainer(profiler=profiler)
+    trainer = Trainer(profiler=SimpleProfiler())
 
     # advanced profiler for function-level stats
-    profiler = AdvancedProfiler()
-    trainer = Trainer(profiler=profiler)
+    trainer = Trainer(profiler=AdvancedProfiler())
 
 progress_bar_refresh_rate
 ^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -738,7 +778,7 @@ How often to refresh progress bar (in steps).
 In notebooks, faster refresh rates (lower number) is known to crash them
 because of their screen refresh rates, so raise it to 50 or more.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(progress_bar_refresh_rate=1)
@@ -770,14 +810,14 @@ replace_sampler_ddp
 ^^^^^^^^^^^^^^^^^^^
 Enables auto adding of distributed sampler.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(replace_sampler_ddp=True)
 
 By setting to False, you have to add your own distributed sampler:
 
-Example::
+.. code-block:: python
 
     # default used by the Trainer
     sampler = torch.utils.data.distributed.DistributedSampler(dataset, shuffle=True)
@@ -787,7 +827,7 @@ resume_from_checkpoint
 ^^^^^^^^^^^^^^^^^^^^^^
 To resume training from a specific checkpoint pass in the path here.
 
-Example::
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(resume_from_checkpoint=None)
@@ -800,17 +840,10 @@ row_log_interval
 
 How often to add logging rows (does not write to disk)
 
-Example::
+.. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(row_log_interval=10)
-
-
-add_row_log_interval:
-
-.. warning:: .. deprecated:: 0.5.0
-
-    Use `row_log_interval` instead. Will remove 0.8.0.
+    trainer = Trainer(row_log_interval=50)
 
 use_amp:
 
@@ -825,42 +858,20 @@ show_progress_bar
 
     Set `progress_bar_refresh_rate` to 0 instead. Will remove 0.9.0.
 
+val_percent_check
+^^^^^^^^^^^^^^^^^
+
+.. warning:: deprecated in v0.8.0 please use `limit_val_batches`. Will remove in 0.10.0
+
 test_percent_check
 ^^^^^^^^^^^^^^^^^^
 
-How much of test dataset to check.
+.. warning:: deprecated in v0.8.0 please use `limit_test_batches`. Will remove in 0.10.0
 
-Example::
+train_percent_check
+^^^^^^^^^^^^^^^^^^^
 
-    # default used by the Trainer
-    trainer = Trainer(test_percent_check=1.0)
-
-    # run through only 25% of the test set each epoch
-    trainer = Trainer(test_percent_check=0.25)
-
-val_check_interval
-^^^^^^^^^^^^^^^^^^
-
-How often within one training epoch to check the validation set.
-Can specify as float or int.
-
-- use (float) to check within a training epoch
-- use (int) to check every n steps (batches)
-
-.. code-block:: python
-
-    # default used by the Trainer
-    trainer = Trainer(val_check_interval=1.0)
-
-Example::
-
-    # check validation set 4 times during a training epoch
-    trainer = Trainer(val_check_interval=0.25)
-
-    # check validation set every 1000 training batches
-    # use this when using iterableDataset and your dataset has no length
-    # (ie: production cases with streaming data)
-    trainer = Trainer(val_check_interval=1000)
+.. warning:: deprecated in v0.8.0 please use `limit_train_batches`. Will remove in 0.10.0
 
 track_grad_norm
 ^^^^^^^^^^^^^^^
@@ -868,34 +879,35 @@ track_grad_norm
 - no tracking (-1)
 - Otherwise tracks that norm (2 for 2-norm)
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(track_grad_norm=-1)
 
-Example::
-
     # track the 2-norm
     trainer = Trainer(track_grad_norm=2)
 
-train_percent_check
+limit_train_batches
 ^^^^^^^^^^^^^^^^^^^
 
 How much of training dataset to check.
 Useful when debugging or testing something that happens at the end of an epoch.
 
-.. code-block::python
+.. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(train_percent_check=1.0)
+    trainer = Trainer(limit_train_batches=1.0)
 
 Example::
 
     # default used by the Trainer
-    trainer = Trainer(train_percent_check=1.0)
+    trainer = Trainer(limit_train_batches=1.0)
 
     # run through only 25% of the training set each epoch
-    trainer = Trainer(train_percent_check=0.25)
+    trainer = Trainer(limit_train_batches=0.25)
+
+    # run through only 10 batches of the training set each epoch
+    trainer = Trainer(limit_train_batches=10)
 
 truncated_bptt_steps
 ^^^^^^^^^^^^^^^^^^^^
@@ -910,7 +922,7 @@ and the trainer will apply Truncated Backprop to it.
 recurrent network trajectories."
 <http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.56.7941&rep=rep1&type=pdf>`_)
 
-Example::
+.. testcode::
 
     # default used by the Trainer (ie: disabled)
     trainer = Trainer(truncated_bptt_steps=None)
@@ -947,47 +959,55 @@ with the hidden
 To modify how the batch is split,
 override :meth:`pytorch_lightning.core.LightningModule.tbptt_split_batch`:
 
-.. code-block:: python
+.. testcode::
 
-        class LitMNIST(pl.LightningModule):
+        class LitMNIST(LightningModule):
             def tbptt_split_batch(self, batch, split_size):
                 # do your own splitting on the batch
                 return splits
 
+val_check_interval
+^^^^^^^^^^^^^^^^^^
 
-val_percent_check
-^^^^^^^^^^^^^^^^^
+How often within one training epoch to check the validation set.
+Can specify as float or int.
 
-How much of validation dataset to check.
-Useful when debugging or testing something that happens at the end of an epoch.
+- use (float) to check within a training epoch
+- use (int) to check every n steps (batches)
 
-Example::
+.. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(val_percent_check=1.0)
+    trainer = Trainer(val_check_interval=1.0)
+
+    # check validation set 4 times during a training epoch
+    trainer = Trainer(val_check_interval=0.25)
+
+    # check validation set every 1000 training batches
+    # use this when using iterableDataset and your dataset has no length
+    # (ie: production cases with streaming data)
+    trainer = Trainer(val_check_interval=1000)
 
-    # run through only 25% of the validation set each epoch
-    trainer = Trainer(val_percent_check=0.25)
 
 weights_save_path
 ^^^^^^^^^^^^^^^^^
 Directory of where to save weights if specified.
 
-.. code-block:: python
+.. testcode::
 
     # default used by the Trainer
     trainer = Trainer(weights_save_path=os.getcwd())
 
-Example::
-
     # save to your custom path
     trainer = Trainer(weights_save_path='my/path')
 
+Example::
+
     # if checkpoint callback used, then overrides the weights path
     # **NOTE: this saves weights to some/path NOT my/path
-    checkpoint_callback = ModelCheckpoint(filepath='some/path')
+    checkpoint = ModelCheckpoint(filepath='some/path')
     trainer = Trainer(
-        checkpoint_callback=checkpoint_callback,
+        checkpoint_callback=checkpoint,
         weights_save_path='my/path'
     )
 
@@ -996,14 +1016,14 @@ weights_summary
 Prints a summary of the weights when training begins.
 Options: 'full', 'top', None.
 
-Example::
+.. testcode::
 
-    # default used by the Trainer (ie: print all weights)
-    trainer = Trainer(weights_summary='full')
-
-    # print only the top level modules
+    # default used by the Trainer (ie: print summary of top level modules)
     trainer = Trainer(weights_summary='top')
 
+    # print full summary of all modules and submodules
+    trainer = Trainer(weights_summary='full')
+
     # don't print a summary
     trainer = Trainer(weights_summary=None)
 
@@ -1013,6 +1033,6 @@ Trainer class
 """
 
 from pytorch_lightning.trainer.trainer import Trainer
-from pytorch_lightning.trainer.seed import seed_everything
+from pytorch_lightning.utilities.seed import seed_everything
 
 __all__ = ['Trainer', 'seed_everything']
