@@ -1,6 +1,6 @@
 import os
 from abc import ABC, abstractmethod
-from typing import Union, List
+from typing import List, Callable, Optional
 
 
 from pytorch_lightning.callbacks import Callback, ModelCheckpoint, EarlyStopping, ProgressBarBase, ProgressBar
@@ -14,12 +14,10 @@ class TrainerCallbackConfigMixin(ABC):
     #  the proper values/initialisation should be done in child class
     callbacks: List[Callback]
     default_root_dir: str
-    logger: Union[LightningLoggerBase, bool]
-    weights_save_path: str
+    logger: LightningLoggerBase
+    weights_save_path: Optional[str]
     ckpt_path: str
-    checkpoint_callback: ModelCheckpoint
-    progress_bar_refresh_rate: int
-    process_position: int
+    checkpoint_callback: Optional[ModelCheckpoint]
 
     @property
     @abstractmethod
@@ -30,6 +28,10 @@ class TrainerCallbackConfigMixin(ABC):
     def save_checkpoint(self, *args):
         """Warning: this is just empty shell for code implemented in other class."""
 
+    @abstractmethod
+    def is_overridden(self, *args):
+        """Warning: this is just empty shell for code implemented in other class."""
+
     def configure_checkpoint_callback(self):
         """
         Weight path set in this priority:
@@ -109,7 +111,7 @@ class TrainerCallbackConfigMixin(ABC):
             self.early_stop_callback = early_stop_callback
             self.enable_early_stop = True
 
-    def configure_progress_bar(self):
+    def configure_progress_bar(self, refresh_rate=1, process_position=0):
         progress_bars = [c for c in self.callbacks if isinstance(c, ProgressBarBase)]
         if len(progress_bars) > 1:
             raise MisconfigurationException(
@@ -117,12 +119,14 @@ class TrainerCallbackConfigMixin(ABC):
                 ' progress bar is supported.'
             )
         elif len(progress_bars) == 1:
-            self.progress_bar_callback = progress_bars[0]
-        elif self.progress_bar_refresh_rate > 0:
-            self.progress_bar_callback = ProgressBar(
-                refresh_rate=self.progress_bar_refresh_rate,
-                process_position=self.process_position,
+            progress_bar_callback = progress_bars[0]
+        elif refresh_rate > 0:
+            progress_bar_callback = ProgressBar(
+                refresh_rate=refresh_rate,
+                process_position=process_position,
             )
-            self.callbacks.append(self.progress_bar_callback)
+            self.callbacks.append(progress_bar_callback)
         else:
-            self.progress_bar_callback = None
+            progress_bar_callback = None
+
+        return progress_bar_callback
