@@ -108,6 +108,19 @@ class TrainerOptimizersMixin(ABC):
                                  'is a invalid input.')
         return lr_schedulers
 
+    def reinit_scheduler_properties(self, optimizers: list, schedulers: list):
+        # Reinitialize optimizer.step properties added by schedulers
+        for scheduler in schedulers:
+            for optimizer in optimizers:
+                scheduler = scheduler['scheduler']
+                # check that we dont mix users optimizers and schedulers
+                if scheduler.optimizer == optimizer:
+                    # Find the mro belonging to the base lr scheduler class
+                    for i, mro in enumerate(scheduler.__class__.__mro__):
+                        if mro == optim.lr_scheduler._LRScheduler:
+                            idx = i
+                    scheduler.__class__.__mro__[idx].__init__(scheduler, optimizer)
+
 
 class _MockOptimizer(Optimizer):
     """The `_MockOptimizer` will be used inplace of an optimizer in the event that `None`
