@@ -25,11 +25,13 @@ class TrainerTrainingTricksMixin(ABC):
     # this is just a summary on variables used in this abstract class,
     #  the proper values/initialisation should be done in child class
     gradient_clip_val: ...
-    precision: ...
+    precision: int
+    default_root_dir: str
+    progress_bar_callback: ...
     on_gpu: bool
 
     @abstractmethod
-    def get_model(self):
+    def get_model(self) -> LightningModule:
         """Warning: this is just empty shell for code implemented in other class."""
 
     @abstractmethod
@@ -133,7 +135,7 @@ class TrainerTrainingTricksMixin(ABC):
                algorithm is terminated
 
         """
-        if not hasattr(model.hparams, batch_arg_name):
+        if not hasattr(model, batch_arg_name):
             raise MisconfigurationException(f'Field {batch_arg_name} not found in `model.hparams`')
 
         if hasattr(model.train_dataloader, 'patch_loader_code'):
@@ -188,7 +190,7 @@ class TrainerTrainingTricksMixin(ABC):
             'early_stop_callback': self.early_stop_callback,
             'enable_early_stop': self.enable_early_stop,
             'auto_scale_batch_size': self.auto_scale_batch_size,
-            'train_percent_check': self.train_percent_check,
+            'limit_train_batches': self.limit_train_batches,
             'model': self.model,
         }
 
@@ -201,7 +203,7 @@ class TrainerTrainingTricksMixin(ABC):
         self.checkpoint_callback = False  # required for saving
         self.early_stop_callback = None
         self.enable_early_stop = False
-        self.train_percent_check = 1.0
+        self.limit_train_batches = 1.0
         self.optimizers, self.schedulers = [], []  # required for saving
         self.model = model  # required for saving
 
@@ -214,7 +216,7 @@ class TrainerTrainingTricksMixin(ABC):
         self.auto_scale_batch_size = self.__dumped_params['auto_scale_batch_size']
         self.early_stop_callback = self.__dumped_params['early_stop_callback']
         self.enable_early_stop = self.__dumped_params['enable_early_stop']
-        self.train_percent_check = self.__dumped_params['train_percent_check']
+        self.limit_train_batches = self.__dumped_params['limit_train_batches']
         self.model = self.__dumped_params['model']
         del self.__dumped_params
 
@@ -243,9 +245,9 @@ def _adjust_batch_size(trainer,
 
     """
     model = trainer.get_model()
-    batch_size = getattr(model.hparams, batch_arg_name)
+    batch_size = getattr(model, batch_arg_name)
     if value:
-        setattr(model.hparams, batch_arg_name, value)
+        setattr(model, batch_arg_name, value)
         new_size = value
         if desc:
             log.info(f'Batch size {batch_size} {desc}, trying batch size {new_size}')
@@ -253,7 +255,7 @@ def _adjust_batch_size(trainer,
         new_size = int(batch_size * factor)
         if desc:
             log.info(f'Batch size {batch_size} {desc}, trying batch size {new_size}')
-        setattr(model.hparams, batch_arg_name, new_size)
+        setattr(model, batch_arg_name, new_size)
     return new_size
 
 
